﻿namespace WebApplication.Controllers
{
    using CelloSaaS.AuthServer.Client.Models;
    using CelloSaaS.Library;
    using CelloSaaS.Model.SettingsManagement;
    using CelloSaaS.ServiceContracts.AccessControlManagement;
    using CelloSaaS.ServiceProxies.SettingsManagement;
    using CelloSaaS.ServiceProxies.TenantManagement;
    using CelloSaaS.View;
    using System;
    using System.Linq;
    using System.Runtime.CompilerServices;
    using System.Threading.Tasks;
    using System.Web.Mvc;

    public class AuthSettingsController : Controller
    {
        #region Members
        private const string DefaultPolicy = "GlobalExceptionLogger";
        private const string ActiveDirectoryIdpKey = "ActiveDirectoryIdp";
        private static string NativeAdf = "eba63b11-c36c-4747-9c7f-a04197dd8026";
        private static CelloSaaS.Library.Encryption.IPasswordEncrptionService encryptionSvc = ServiceLocator.Resolve<CelloSaaS.Library.Encryption.IPasswordEncrptionService>();

        #endregion

        #region Active Directory Based Tenant Settings

        [HttpGet]
        public ActionResult Index()
        {
            return View("Settings");
        }

        [HttpPost]
        public JsonResult GetChildTenants()
        {
            try
            {
                var childTenants = TenantRelationProxy.GetAllActiveImmediateChildTenants(UserIdentity.TenantID);

                if (childTenants == null || childTenants.Count < 1)
                    return JsonResponse(Status.Error, Resources.TenantResource.m_ChildTenantsNotFound, new { });

                return JsonResponse(Status.Success, null, childTenants.Select(x => new { x.Key, x.Value.TenantName }).ToList());
            }
            catch (Exception exception)
            {
                Task.Run(() => ExceptionService.HandleException(exception, DefaultPolicy));
                return JsonResponse(Status.Error, Resources.TenantResource.m_ChildTenantsNotFound, new { });
            }
        }

        [HttpPost]
        public JsonResult GetSettings(string tenantId)
        {
            if (string.IsNullOrEmpty(tenantId))
                return JsonResponse(Status.Error, Resources.TenantResource.e_TenantIdentificationFailed, new { });

            try
            {
                var setting = TenantSettingsProxy.GetTenantSettings(tenantId);

                if (setting != null && setting.Setting != null && setting.Setting.Attributes != null && setting.Setting.Attributes.Count > 0)
                {
                    //defaults to cello openid
                    string idp = AppSettingHelper.CelloOpenIdentity;
                    string authType = AppSettingHelper.CelloAuthentication;

                    AuthorizationSettings settings = null;

                    // for pre-existing tenant settings
                    if (setting.Setting.Attributes.ContainsKey(AuthServerSettingConstants.AuthorizationSettings))
                    {
                        var data = setting.Setting.Attributes[AuthServerSettingConstants.AuthorizationSettings];

                        settings = string.IsNullOrEmpty(data)
                                    ? null
                                    : Newtonsoft.Json.JsonConvert.DeserializeObject<AuthorizationSettings>(data);
                    }

                    //override if required for IDP & AuthTypes
                    if (setting.Setting.Attributes.ContainsKey(AuthServerSettingConstants.IdentityProviders))
                        idp = setting.Setting.Attributes[AuthServerSettingConstants.IdentityProviders];

                    if (setting.Setting.Attributes.ContainsKey(AuthServerSettingConstants.DefaultAuthenticationType))
                        authType = setting.Setting.Attributes[AuthServerSettingConstants.DefaultAuthenticationType];

                    string claimMapSetting = setting.Setting.Attributes.ContainsKey(AuthServerSettingConstants.ClaimMap)
                                                ? setting.Setting.Attributes[AuthServerSettingConstants.ClaimMap]
                                                : string.Empty;

                    var authSettings = InitializeSettings(tenantId, idp, authType, settings, claimMapSetting);

                    // Fill-in the client id / secret 
                    if (setting.Setting.Attributes.ContainsKey(AuthServerSettingConstants.ClientId)
                        && !string.IsNullOrEmpty(setting.Setting.Attributes[AuthServerSettingConstants.ClientId]))
                    {
                        authSettings.ClientId = SecureValue(setting.Setting.Attributes[AuthServerSettingConstants.ClientId], SecurityType.Decrypt);
                    }

                    if (setting.Setting.Attributes.ContainsKey(AuthServerSettingConstants.ClientSecret)
                        && !string.IsNullOrEmpty(setting.Setting.Attributes[AuthServerSettingConstants.ClientSecret]))
                    {
                        authSettings.ClientSecret = SecureValue(setting.Setting.Attributes[AuthServerSettingConstants.ClientSecret], SecurityType.Decrypt);
                    }

                    return JsonResponse(Status.Success, Resources.TenantResource.m_EndpointsFetchSuccess, authSettings);
                }
                return JsonResponse(Status.Error, Resources.TenantResource.e_SettingsNotFoundTypeMismatch, new { });
            }
            catch (Exception exception)
            {
                ExceptionService.HandleException(exception, DefaultPolicy);
                return JsonResponse(Status.Error, Resources.TenantResource.e_EndpointException, new { });
            }
        }

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        private static TenantAuthorizationSettings InitializeSettings(string tenantId, string idp, string authType, AuthorizationSettings settings, string claimMapSetting)
        {
            var authSettings = new TenantAuthorizationSettings
            {
                AuthorizationSettings = new ADFSAuthorizationSettings
                {
                    AuthorizationEndpointUri = null,
                    TokenEndpointUri = null,
                    UserInfoEndpointUri = null,
                    RequireFedMetadata = string.IsNullOrEmpty(authType)
                                            ? false
                                            : authType.Equals(NativeAdf, StringComparison.OrdinalIgnoreCase),

                },
                ClaimMapType = claimMapSetting,
                TenantId = tenantId
            };

            if (settings == null && !idp.Equals(AppSettingHelper.ActiveDirectoryIdp, StringComparison.OrdinalIgnoreCase))
            {
                authSettings.AuthorizationSettings = null;
                return authSettings;
            }

            if (settings == null) settings = authSettings.AuthorizationSettings;

            authSettings.AuthorizationSettings.AuthorizationEndpointUri = settings.AuthorizationEndpointUri;
            authSettings.AuthorizationSettings.TokenEndpointUri = settings.TokenEndpointUri;
            authSettings.AuthorizationSettings.UserInfoEndpointUri = settings.UserInfoEndpointUri;

            return authSettings;
        }

        [HttpPost]
        public async Task<JsonResult> SaveSettings(TenantAuthorizationSettings tenantAuthorizationSettings)
        {
            if (tenantAuthorizationSettings == null)
                return JsonResponse(Status.Error, Resources.TenantResource.e_TenantAuthSettingNull, new { });

            try
            {
                if (string.IsNullOrEmpty(tenantAuthorizationSettings.TenantId))
                {
                    return JsonResponse(Status.Error, Resources.TenantResource.e_TenantAuthSettingNull, new { });
                }

                // get the current tenant setting
                TenantSetting tenantSetting = TenantSettingsProxy.GetTenantSettings(tenantAuthorizationSettings.TenantId);

                if (tenantSetting == null || tenantSetting.Setting == null || tenantSetting.Setting.Attributes == null || tenantSetting.Setting.Attributes.Count < 1
                    || !tenantSetting.Setting.Attributes.ContainsKey(AuthServerSettingConstants.IdentityProviders)
                    || !tenantSetting.Setting.Attributes.ContainsKey(AuthServerSettingConstants.DefaultAuthenticationType))
                {

                    if (!tenantSetting.Setting.Attributes.ContainsKey(AuthServerSettingConstants.IdentityProviders))
                    {
                        tenantSetting.Setting.Attributes.Add(AuthServerSettingConstants.IdentityProviders, AppSettingHelper.CelloOpenIdentity);
                    }
                    if (!tenantSetting.Setting.Attributes.ContainsKey(AuthServerSettingConstants.DefaultAuthenticationType))
                    {
                        tenantSetting.Setting.Attributes.Add(AuthServerSettingConstants.DefaultAuthenticationType, AppSettingHelper.CelloAuthentication);
                    }
                }
                var settings = tenantAuthorizationSettings.AuthorizationSettings;
                string tenantIdp = tenantSetting.Setting.Attributes[AuthServerSettingConstants.IdentityProviders];

                if (tenantIdp.Equals(AppSettingHelper.ActiveDirectoryIdp, StringComparison.OrdinalIgnoreCase) && (settings == null || (string.IsNullOrEmpty(settings.AuthorizationEndpointUri) || string.IsNullOrEmpty(settings.TokenEndpointUri))))
                {
                    return JsonResponse(Status.Error, Resources.TenantResource.e_EndpointNull, new { });
                }

                if (tenantIdp.Equals(AppSettingHelper.ActiveDirectoryIdp, StringComparison.OrdinalIgnoreCase))
                {
                    if (tenantSetting.Setting.Attributes.ContainsKey(AuthServerSettingConstants.AuthorizationSettings))
                    {
                        tenantSetting.Setting.Attributes.Remove(AuthServerSettingConstants.AuthorizationSettings);
                    }

                    var authTypeId = tenantSetting.Setting.Attributes[AuthServerSettingConstants.DefaultAuthenticationType];

                    // ADFS Native => If Federation Metadata is available, save it
                    if (authTypeId.Equals(NativeAdf, StringComparison.OrdinalIgnoreCase) && !string.IsNullOrEmpty(settings.FederationMetadataUrl))
                    {
                        tenantSetting.Setting.Attributes.Add(AuthServerSettingConstants.FederationMetadataUri, settings.FederationMetadataUrl);
                    }

                    var authorizationSettings = new AuthorizationSettings
                    {
                        AuthorizationEndpointUri = settings.AuthorizationEndpointUri,
                        TokenEndpointUri = settings.TokenEndpointUri,
                        UserInfoEndpointUri = settings.UserInfoEndpointUri
                    };
                    tenantSetting.Setting.Attributes.Add(AuthServerSettingConstants.AuthorizationSettings, Newtonsoft.Json.JsonConvert.SerializeObject(authorizationSettings));
                }

                if (tenantSetting.Setting.Attributes.ContainsKey(AuthServerSettingConstants.ClientId))
                {
                    tenantSetting.Setting.Attributes.Remove(AuthServerSettingConstants.ClientId);
                }
                if (tenantSetting.Setting.Attributes.ContainsKey(AuthServerSettingConstants.ClientSecret))
                {
                    tenantSetting.Setting.Attributes.Remove(AuthServerSettingConstants.ClientSecret);
                }

                if (tenantSetting.Setting.Attributes.ContainsKey(AuthServerSettingConstants.ClaimMap))
                {
                    tenantSetting.Setting.Attributes.Remove(AuthServerSettingConstants.ClaimMap);
                }
                if (!string.IsNullOrEmpty(tenantAuthorizationSettings.ClientId))
                    tenantSetting.Setting.Attributes.Add(AuthServerSettingConstants.ClientId, SecureValue(tenantAuthorizationSettings.ClientId));
                if (!string.IsNullOrEmpty(tenantAuthorizationSettings.ClientSecret))
                    tenantSetting.Setting.Attributes.Add(AuthServerSettingConstants.ClientSecret, SecureValue(tenantAuthorizationSettings.ClientSecret));

                tenantSetting.Setting.Attributes.Add(AuthServerSettingConstants.ClaimMap, tenantAuthorizationSettings.ClaimMapType);

                TenantSettingsProxy.UpdateTenantSettings(tenantSetting);

                string authServerUri = AppSettingHelper.AuthServerCacheUri;

                var httpClient = new System.Net.Http.HttpClient();
                var response = await httpClient.GetAsync(authServerUri);

                if (!response.IsSuccessStatusCode)
                {
                    LogService.Write("Unable to refresh the AuthServer Cache. Try clearing manually");
                    return JsonResponse(Status.Success, Resources.TenantResource.m_EndPointCacheFailInAuthServer, new { });
                }

                return JsonResponse(Status.Success, Resources.TenantResource.m_EndpointSaveSuccessful, new { });
            }
            catch (Exception exception)
            {
                ExceptionService.HandleException(exception, DefaultPolicy);
                return JsonResponse(Status.Error, Resources.TenantResource.e_EndpointSaveFail, new { });
            }
        }

        #region Private Methods

        internal enum SecurityType : int
        {
            Encrypt = 0,
            Decrypt
        }

        internal static string SecureValue(string input, SecurityType type = SecurityType.Encrypt)
        {
            if (string.IsNullOrEmpty(input)) return null;
            switch (type)
            {
                default:
                case SecurityType.Encrypt:
                    return encryptionSvc.EncryptPassword(ProductAdminConstants.ProductAdminTenantId, input);
                case SecurityType.Decrypt:
                    return encryptionSvc.DecryptPassword(ProductAdminConstants.ProductAdminTenantId, input);
            }
        }

        private JsonResult JsonResponse<T>(Status status, string message, T data, bool denyGet = true)
        {
            return Json(new
            {
                Status = status.ToString("F"),
                Message = message,
                Data = data
            }, denyGet ? JsonRequestBehavior.DenyGet : JsonRequestBehavior.AllowGet);
        }

        public enum Status : short
        {
            Success = 0,
            Error
        }
        #endregion

        #endregion
    }
}