﻿using CelloSaaS.View;
using Newtonsoft.Json;
using System;
using System.Linq;
using System.Collections.Generic;
using System.Dynamic;
using System.Globalization;
using System.Net.Http;
using System.Web;
using CelloSaaS.AuthServer.Client.Extensions;
using CelloSaaSWebClient.Models;
using CelloSaaS.AuthServer.Extensions.Services;
using System.Data.Common;
using CelloSaaS.AuthServer.Extensions.Models;
using CelloSaaS.Library;
using System.Threading.Tasks;

namespace CelloSaaSWebClient.Services
{
    /// <summary>
    /// The authentication service that will be used to get the identity and the authentication providers from the Authorization Server
    /// </summary>
    public static class AuthenticationService
    {
        private const string LoggingPolicy = "GlobalExceptionLogger";
        private const string CacheKey = "AuthDetailsKey_", Providers = "Providers", Types = "Types";

        private static readonly AuthenticationTypesService authService = null;
        static AuthenticationService()
        {
            authService = new AuthenticationTypesService();
        }

        /// <summary>
        /// This method is used to get the various registered authentication providers
        /// </summary>
        /// <returns>Collection of authentication providers </returns>
        public static Dictionary<string, string> GetAuthenticationProviders()
        {
            try
            {
                Dictionary<string, string> providers = null; //CacheManager.GetData(CacheKey + Providers) as Dictionary<string, string>;
                if (providers != null && providers.Count > 0)
                    return providers;

                var authProviders = authService.GetAuthenticationProviders(null);

                if (authProviders == null || authProviders.Count < 1) return null;

                providers = authProviders.OrderBy(o => o.Value.Name).ToDictionary(t => t.Value.Id.ToString(), t => t.Value.Name);

                //CacheManager.AddToCache(CacheKey + Providers, providers);

                return providers;
            }
            catch (DbException exception)
            {
                Task.Run(() => CelloSaaS.Library.ExceptionService.HandleException(exception, LoggingPolicy));
                return null;
            }
        }

        /// <summary>
        /// This method is used to get the authentication types for the provider id if passed as an argument
        /// </summary>
        /// <param name="providerId">The authentication provider identifier</param>
        /// <returns>Collection of Authentication types grouped by the provider identifiers</returns>
        public static Dictionary<string, Dictionary<string, string>> GetAuthenticationTypes(string providerId)
        {
            try
            {
                Dictionary<string, AuthenticationType> authenticationTypes = null; //CacheManager.GetData(CacheKey + Types) as Dictionary<string, AuthenticationType>;

                if (authenticationTypes == null || authenticationTypes.Count < 1)
                {
                    authenticationTypes = new AuthenticationTypesService().GetAuthenticationTypes(string.IsNullOrEmpty(providerId) ? null : new string[] { providerId });

                    if (authenticationTypes == null || authenticationTypes.Count < 1) return null;

                    //CacheManager.AddToCache(CacheKey + Types, authenticationTypes);
                }

                Dictionary<string, AuthenticationType> authTypes = null;

                authTypes = (string.IsNullOrEmpty(providerId))
                            ? new AuthenticationTypesService().GetAuthenticationTypes(null)
                            : new AuthenticationTypesService().GetAuthenticationTypes(new string[] { providerId });

                Dictionary<string, Dictionary<string, string>> authTypesResult = new Dictionary<string, Dictionary<string, string>>(StringComparer.OrdinalIgnoreCase);

                foreach (var authType in authTypes)
                {
                    if (!authTypesResult.ContainsKey(authType.Value.ProviderId.ToString()))
                        authTypesResult.Add(authType.Value.ProviderId.ToString().ToLowerInvariant(), new Dictionary<string, string>(StringComparer.OrdinalIgnoreCase));

                    if (!authTypesResult[authType.Value.ProviderId.ToString()].ContainsKey(authType.Value.Id.ToString()))
                        authTypesResult[authType.Value.ProviderId.ToString()].Add(authType.Value.Id.ToString().ToLowerInvariant(), authType.Value.Name);
                }

                return authTypesResult;
            }
            catch (DbException exception)
            {
                Task.Run(() => CelloSaaS.Library.ExceptionService.HandleException(exception, LoggingPolicy));
                return null;
            }
        }
    }
}